# -*- coding: utf-8 -*-
'''
Created on 2019年12月5日

@author: siwenwei
'''


class Tags(object):

    html = "html"

    meta = "meta"

    title = "title"

    style = "style"

    script = "script"

    head = "head"

    body = "body"

    table = "table"

    thead = "thead"

    tbody = "tbody"

    th = "th"

    tr = "tr"

    td = "td"

    div = "div"

    span = "span"

    label = "label"

    fieldset = "fieldset"

    legend = "legend"

    pre = "pre"

    svg = "svg"

    path = "path"

    text = "text"

    rect = "rect"

    circle = "circle"

    dl = "dl"

    dt = "dt"

    dd = "dd"

    input = "input"

    ul = "ul"

    li = "li"

    textarea = "textarea"

    italic = "i"

    p = "p"

    hr = "hr"
